"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const oauth_1 = require("../../google/oauth");
const gmail_1 = require("../../google/gmail");
const snapshot_1 = require("../../data/snapshot");
class CreateSnapshot extends core_1.Command {
    async run() {
        var _a;
        const gmails = (await (0, oauth_1.prepareOAuthWithCredentials)()).map(gmail_1.toGmail);
        const snapshot = new snapshot_1.SnapshotBuilder();
        for (const gmail of gmails) {
            const progress = core_1.CliUx.ux.progress({ noTTYOutput: true });
            const profile = await gmail.getProfile();
            if (!profile.emailAddress) {
                throw new Error('emailAddress must be defined');
            }
            core_1.CliUx.ux.info(`creating snapshot for ${profile.emailAddress}`);
            const snapshotWriter = await snapshot.getSnapshotWriter(profile.emailAddress, count => progress.update(count));
            const labels = await gmail.getLabels();
            await snapshotWriter.writeLabels(labels);
            progress.start();
            const lastSavedMessage = await (0, snapshot_1.findLastSavedMessage)(profile.emailAddress);
            const emailReader = gmail.getEmailReader(total => progress.setTotal(total), (_a = lastSavedMessage === null || lastSavedMessage === void 0 ? void 0 : lastSavedMessage.id) !== null && _a !== void 0 ? _a : undefined);
            await emailReader.pipeTo(snapshotWriter.messageWithAttachments);
            progress.stop();
        }
        core_1.CliUx.ux.info('snapshot created');
    }
}
exports.default = CreateSnapshot;
CreateSnapshot.description = 'Creates new incremental snapshot in configured directory. ' +
    'It\'s meant to be run periodically (for example with cron).';
