"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toGmail = exports.Gmail = void 0;
const googleapis_1 = require("googleapis");
const web_1 = require("node:stream/web");
const common_1 = require("../common");
const LIST_PARAMS = { userId: 'me', maxResults: 500 };
class Gmail {
    constructor(gmail) {
        this.gmail = gmail;
    }
    async getProfile() {
        const { data: profile } = await this.gmail.users.getProfile({ userId: 'me' });
        return profile;
    }
    getEmailReader(setTotal, lastMessageId) {
        let total = 0;
        const mailIdsStream = new web_1.ReadableStream({
            start: async (controller) => {
                var _a, _b, _c, _d, _e, _f, _g;
                let nextPageToken;
                do {
                    const { data: mailIds } = await this.gmail.users.messages.list({ ...LIST_PARAMS, pageToken: nextPageToken });
                    nextPageToken = (_a = mailIds.nextPageToken) !== null && _a !== void 0 ? _a : undefined;
                    const lastMessageIndex = (_c = (_b = mailIds.messages) === null || _b === void 0 ? void 0 : _b.findIndex(message => message.id === lastMessageId)) !== null && _c !== void 0 ? _c : -1;
                    if (lastMessageIndex !== -1) {
                        mailIds.messages = (_d = mailIds.messages) === null || _d === void 0 ? void 0 : _d.slice(0, lastMessageIndex);
                        nextPageToken = undefined;
                    }
                    (_e = mailIds.messages) === null || _e === void 0 ? void 0 : _e.forEach(message => controller.enqueue(message.id));
                    total += (_g = (_f = mailIds.messages) === null || _f === void 0 ? void 0 : _f.length) !== null && _g !== void 0 ? _g : 0;
                    setTotal === null || setTotal === void 0 ? void 0 : setTotal(total);
                } while (nextPageToken);
                controller.close();
            },
        });
        const fetchMailsStream = new web_1.TransformStream({
            transform: async (messageId, controller) => {
                const { data: message } = await this.gmail.users.messages.get({ userId: 'me', id: messageId });
                controller.enqueue(message);
            },
        }, common_1.QUEUING_STRATEGY, common_1.QUEUING_STRATEGY);
        const fetchAttachmentsStream = new web_1.TransformStream({
            transform: async (message, controller) => {
                var _a, _b;
                const attachments = await Promise.all(((_b = (_a = message.payload) === null || _a === void 0 ? void 0 : _a.parts) === null || _b === void 0 ? void 0 : _b.filter(part => {
                    var _a;
                    return (_a = part.headers) === null || _a === void 0 ? void 0 : _a.some(header => { var _a; return header.name === 'Content-Disposition' && ((_a = header.value) === null || _a === void 0 ? void 0 : _a.startsWith('attachment')); });
                }).map(part => part.body.attachmentId).map(attachmentId => this.gmail.users.messages.attachments.get({
                    userId: 'me',
                    messageId: message.id,
                    id: attachmentId,
                }).then(response => ({ attachmentId, ...response.data })))) || []);
                controller.enqueue({ message, attachments });
            },
        });
        return mailIdsStream
            .pipeThrough((0, common_1.reverseStream)())
            .pipeThrough(fetchMailsStream)
            .pipeThrough(fetchAttachmentsStream);
    }
    async getLabels() {
        var _a;
        const response = await this.gmail.users.labels.list({ userId: 'me' });
        return (_a = response.data.labels) !== null && _a !== void 0 ? _a : [];
    }
}
exports.Gmail = Gmail;
function toGmail(auth) {
    const gmail = googleapis_1.google.gmail({ version: 'v1', auth });
    return new Gmail(gmail);
}
exports.toGmail = toGmail;
