"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.prepareOAuthWithCredentials = exports.prepareCredentials = exports.generateAuthUrl = exports.setupOAuth = void 0;
const tslib_1 = require("tslib");
const googleapis_1 = require("googleapis");
const fs = tslib_1.__importStar(require("node:fs/promises"));
const credentials_1 = require("../data/credentials");
const config_1 = require("../config");
let oAuth2Client;
let clientConfig;
function prepareOAuth() {
    return new googleapis_1.Auth.OAuth2Client({
        clientId: clientConfig.installed.client_id,
        clientSecret: clientConfig.installed.client_secret,
        redirectUri: clientConfig.installed.redirect_uris[0],
    });
}
async function setupOAuth() {
    const buffer = await fs.readFile(`${(0, config_1.getConfig)().secretsPath}/client_secret.json`);
    clientConfig = JSON.parse(buffer.toString());
    oAuth2Client = prepareOAuth();
}
exports.setupOAuth = setupOAuth;
const GENERATE_URL_OPTS = { access_type: 'offline', scope: ['https://www.googleapis.com/auth/gmail.readonly'] };
function generateAuthUrl() {
    return oAuth2Client.generateAuthUrl(GENERATE_URL_OPTS);
}
exports.generateAuthUrl = generateAuthUrl;
async function prepareCredentials(url) {
    const urlSearchParams = new URLSearchParams(url.split('?')[1]);
    const code = urlSearchParams.get('code');
    if (!code) {
        throw new Error('invalid URL');
    }
    const response = await oAuth2Client.getToken(code);
    return response.tokens;
}
exports.prepareCredentials = prepareCredentials;
async function prepareOAuthWithCredentials() {
    const credentials = await (0, credentials_1.getCredentials)();
    return credentials.map(credential => {
        const oAuth2Client = prepareOAuth();
        oAuth2Client.setCredentials(credential);
        return oAuth2Client;
    });
}
exports.prepareOAuthWithCredentials = prepareOAuthWithCredentials;
